#include <stdio.h>
enum EuTypes {INTEGER, ATOM, SEQUENCE};
void doError(char *msg)
{
   printf("%s",msg);
   exit(1);
}
class EuObject
{
 protected:
           union
           {
               double atomValue;
               int integerValue;
               EuObject *first;
           };
           EuObject *next;
           EuTypes type;
 public:
        EuObject(EuObject *o);
        EuObject(double v);
        EuObject(int i);
        EuObject();
        int getIntValue();
        double getAtomValue();
        void setValue(int v);
        void setValue(double v);
        void setValue(EuObject *v);
        EuObject *getSubscript(int s);
        void setSubscript(int s, EuObject *o);
        void addItem(EuObject *obj);
        void addItem(int i);
        void addItem(double v);
        int getLength();
        void setType(EuTypes t);
        EuTypes getType();
        EuObject *getSubscripts(int s, int e);
        void setSubscripts(int s, int e, EuObject *v);
        void setRepeat(EuObject *obj, int n);
        void clear();
};
EuTypes EuObject::getType()
{
   return type;
}
void EuObject::setType(EuTypes t)
{
   type = t;
}
void EuObject::clear()
{
	type = SEQUENCE;
	EuObject *o = first;
	EuObject *n;
	while (o)
	{
		n = o->next;
		delete o;
		o = n;
	}
	first = 0;
}
int EuObject::getLength()
{
   if (type != SEQUENCE)
   {
      doError("attempting to get length of an atom\n");
   }
   EuObject *o = first;
   int l;
   l = 0;
   while (o)
   {
      l++;
      o = o->next;
   }
   return l;
}
void EuObject::addItem(EuObject *obj)
{
   EuObject *o;
   if (first == 0)
   {
      first = new EuObject(obj);
      first->next = 0;
   } else {
      o = getSubscript(getLength());
      o->next = new EuObject(obj);
      o->next->next = 0;
   }
}
void EuObject::addItem(int obj)
{
   EuObject *o;
   if (first == 0)
   {
      first = new EuObject(obj);
      first->next = 0;
   } else {
      o = getSubscript(getLength());
      o->next = new EuObject(obj);
      o->next->next = 0;
   }
}
void EuObject::addItem(double obj)
{
   EuObject *o;
   if (first == 0)
   {
      first = new EuObject(obj);
      first->next = 0;
   } else {
      o = getSubscript(getLength());
      o->next = new EuObject(obj);
      o->next->next = 0;
   }
}
void EuObject::setValue(EuObject *o)
{
   int i;
   type = o->type;
   if (type == ATOM)
   {
      atomValue = o->atomValue;
   } else if (type == INTEGER)
   {
      integerValue = o->integerValue;
   } else {
     first = 0;
     for (i = 1; i <= o->getLength(); i++)
     {
         addItem(o->getSubscript(i));
     }
   }
}
EuObject::EuObject(EuObject *o)
{
   setValue(o);
}
EuObject::EuObject(double v)
{
   type = ATOM;
   atomValue = v;
}
EuObject::EuObject(int i)
{
   type = INTEGER;
   integerValue = i;
}
EuObject::EuObject()
{
   first = 0;
}
int EuObject::getIntValue()
{
   if (type == INTEGER)
   {
      return integerValue;
   } else if (type == ATOM)
   {
      return (int)atomValue;
   } else {
     return 0;
   }
}
double EuObject::getAtomValue()
{
   if (type == INTEGER)
   {
      return (double)integerValue;
   } else if (type == ATOM)
   {
      return atomValue;
   } else
   {
      return 0;
   }
}
void EuObject::setValue(int v)
{
   type = INTEGER;
   integerValue = v;
}
void EuObject::setValue(double v)
{
   type = ATOM;
   atomValue = v;
}
EuObject *EuObject::getSubscript(int s)
{
   if (type != SEQUENCE)
   {
      doError("attempt to subscript an atom\n");
   }
   EuObject *obj;
   obj = first;
   while (--s)
   {
      obj = obj->next;
   }
   return obj;
}
void EuObject::setSubscript(int s, EuObject *obj)
{
   if (type != SEQUENCE)
   {
      doError("attempt to subscript an atom\n");
   }
   EuObject *sub = getSubscript(s);
   int i;
   sub->type = obj->type;
   if (sub->type == ATOM)
   {
      sub->atomValue = obj->atomValue;
   } else if (sub->type == INTEGER)
   {
      sub->integerValue = obj->integerValue;
   } else {
      sub->first = 0;
      for (i = 1; i <= obj->getLength(); i++)
      {
         sub->addItem(obj->getSubscript(i));
      }
   }
}
EuObject *EuObject::getSubscripts(int s, int e)
{
   if (type != SEQUENCE)
   {
      doError("attempt to subscript an atom\n");
   }
   if (s < 1)
   {
      doError("slice start < 1\n");
   }
   if (e < 1)
   {
      doError("slice end < 1\n");
   }
   if (s > e)
   {
      doError("slice length < 0\n");
   }
   EuObject *obj;
   EuObject *r = new EuObject;
   r->setType(SEQUENCE);
   obj = first;
   while (--s)
   {
      e--;
      if (obj == 0)
      {
         doError("index out of bounds\n");
      }
      obj = obj->next;
   }
   while (e--)
   {
      if (obj == 0)
      {
         doError("index out of bounds\n");
      }
      r->addItem(obj);
      obj = obj->next;
   }
   return r;
}
void EuObject::setSubscripts(int s, int e, EuObject *v)
{
   int l = (e-s)+1;
   if (type != SEQUENCE)
   {
      doError("attempt to subscript an atom\n");
   }
   if (s < 1)
   {
      doError("slice start < 1\n");
   }
   if (e < 1)
   {
      doError("slice end < 1\n");
   }
   if (s > e)
   {
      doError("slice length < 0\n");
   }
   EuObject *o = first;
   while (--s)
   {
      e--;
      if (o == 0)
      {
         doError("index out of bounds\n");
      }
      o = o->next;
   }
   if (v->getType() == SEQUENCE)
   {
      if (v->getLength() != l)
      {
         doError("sequence lengths are not the same\n");
      }
      EuObject *obj = v->first;
      while (e--)
      {
         if (o == 0)
         {
            doError("index out of bounds\n");
         }
         o->setValue(obj);
         obj = obj->next;
         o = o->next;
      }
   } else {
      while (e--)
      {
         if (o == 0)
         {
            doError("index out of bounds\n");
         }
         o->setValue(v);
         o = o->next;
      }
   }
}
void EuObject::setRepeat(EuObject *obj, int n)
{
	clear();
	for (int i = 0; i < n; i++)
	{
		addItem(obj);
	}
}
void add(EuObject *a, EuObject *b, EuObject *c)
{
	c->clear();
	for (int i = 1; i <= a->getLength(); i++)
	{
		c->addItem(a->getIntValue()+b->getIntValue());
	}
}